//---------------------------------------------------------------------------
//
//                      WWW.NATUREWIZARD.COM
//                      ~~~~~~~~~~~~~~~~~~~~
//
//              Author      : Dr. Heinz Seyringer
//              Project     : Tutorial 1 - Text Output
//              Description : This project demonstrates how to use text
//                            in OpenGL
//
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "MAIN.h"
#include "math.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFormMain *FormMain;

GLint nFontList;

//---------------------------------------------------------------------------
__fastcall TFormMain::TFormMain(TComponent* Owner)
    : TForm(Owner)
{
    Application->OnIdle = IdleLoop;   // Define idle function
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::IdleLoop(TObject*, bool& done)
{
    done = false;               // request more idle time from the system
    RenderGLScene();
    SwapBuffers(hdc);
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::SetupText(HDC hdc)
{
        //nFontList is declared as global variable
	// Setup the Font characteristics
	HFONT hFont;
	GLYPHMETRICSFLOAT agmf[128];
	LOGFONT logfont;

	logfont.lfHeight = -12;
	logfont.lfWidth = 0;
	logfont.lfEscapement = 0;
	logfont.lfOrientation = 0;
	logfont.lfWeight = FW_NORMAL;
	logfont.lfItalic = FALSE;
	logfont.lfUnderline = TRUE;
	logfont.lfStrikeOut = FALSE;
	logfont.lfCharSet = ANSI_CHARSET;
	logfont.lfOutPrecision = OUT_DEFAULT_PRECIS;
	logfont.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	logfont.lfQuality = PROOF_QUALITY;
	logfont.lfPitchAndFamily = DEFAULT_PITCH || FF_ROMAN;
	strcpy(logfont.lfFaceName,"Verdana");

	// Create the font and display list
	hFont = CreateFontIndirect(&logfont);
	SelectObject (hdc, hFont);

	nFontList = glGenLists(128);    // create display lists
	wglUseFontOutlines(hdc, 0, 128, nFontList, 0.0f, 0.5f,
				WGL_FONT_POLYGONS, agmf);

	DeleteObject(hFont);
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormCreate(TObject *Sender)
{
    alpha = 0.0;                   // set y-rotation angle to 0
    beta  = 0.0;                   // set x-rotation angle to 0
    hdc = GetDC(Handle);           // get device context from main window
    SetPixelFormatDescriptor();
    hrc = wglCreateContext(hdc);   // use device context to create
                                   // a rendering context
    if(hrc == NULL)
    	ShowMessage("Creating Rendering Context failed.");
    wglMakeCurrent(hdc,hrc);       // tell windows to use hdc and rdc
    if(wglMakeCurrent(hdc, hrc) == false)
    	ShowMessage("MakeCurrent of rendering context failed.");
    w = ClientWidth;               // width of main windows client area
    h = ClientHeight;              // height of main windows client area

    glEnable(GL_DEPTH_TEST);
 //   glEnable(GL_CULL_FACE);      // removed to avoid invisible backside of the text

    glClearColor(0.0f, 0.0f, 0.0f, 1.0f);  // clear background to black
    glClearDepth(100.0);                   // set depth buffer to the most distant value
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    SetupText(hdc);                // prepare text output
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::SetPixelFormatDescriptor()
{
    PIXELFORMATDESCRIPTOR pfd = {
    	sizeof(PIXELFORMATDESCRIPTOR),       // size of the structure
        1,                                   // structure version
        PFD_DRAW_TO_WINDOW |                 // draw directly to window
                                             // (not to a bitmap file)
        PFD_SUPPORT_OPENGL |                 // allow DC to support opengl calls
        PFD_DOUBLEBUFFER,                    // use double buffer
        PFD_TYPE_RGBA,                       // use RGBA color mode (A = alpha)
        24,                                  // use 24 bit color
        0,0,0,0,0,0,0,0,0,0,0,0,0,           // use default bitplanes
        32,                                  // use 32 bit z buffer
        0,0,                                 // not (yet) used
        PFD_MAIN_PLANE,                      // draw to main plane
        0,0,0,                               // different mask bits
    };
    PixelFormat = ChoosePixelFormat(hdc, &pfd);  // choose and set the
    SetPixelFormat(hdc, PixelFormat, &pfd);      // appropriate pixelformat
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormResize(TObject *Sender)
{

    GLfloat Range = 100.0;// numerical range on the 3 axes

    w = ClientWidth;      // check new client width of the main window
    h = ClientHeight;     // check new client height of the main window

    if(h == 0)            // prevent a division by zero, by making sure that
    	h = 1;            // windows height is at least 1

    glViewport(0, 0, w, h);         // reset viewport


    glMatrixMode(GL_PROJECTION);    // add perspective to scene
    glLoadIdentity();               // restore matrix to original state

    if (w <= h)
        glOrtho (-Range, Range, -Range*h/w, Range*h/w, -Range, Range);
    else
	glOrtho (-Range*w/h, Range*w/h, -Range, Range, -Range, Range);

    glMatrixMode(GL_MODELVIEW);
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::RenderGLScene()
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    DrawObjects();
    glFlush();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::DrawObjects()
{

   // rendering functions

   alpha+=0.2;
   beta+=0.1;
   glLoadIdentity();
   glTranslatef( 0.0f, 0.0f, 0.0f);             // move triangle in scene
   glRotatef(alpha,0.0f,1.0f,0.0f);             // rotate around y-axis
   glRotatef(beta,1.0f,0.0f,0.0f);              // rotate around x-axis
   glBegin(GL_TRIANGLE_FAN);                       // draw triangle
      glColor3f(1.0f,0.0f,0.0f);                // set color to red
      glVertex3f(  0.0f,  30.0f, 0.0f);
      glColor3f(0.0f,1.0f,0.0f);                // set color to green
      glVertex3f(-50.0f, -50.0f, 50.0f);
      glColor3f(1.0f,1.0f,0.0f);                // set color to yellow
      glVertex3f( 50.0f, -50.0f, 50.0f);
      glColor3f(0.0f,0.0f,1.0f);                // set color to blue
      glVertex3f( 50.0f, -50.0f, -50.0f);
      glColor3f(1.0f,1.0f,1.0f);                // set color to white
      glVertex3f( -50.0f, -50.0f, -50.0f);
      glColor3f(0.0f,1.0f,0.0f);                // set color to green
      glVertex3f(-50.0f, -50.0f, 50.0f);
   glEnd();
   glBegin(GL_QUADS);                           // draw square
      glColor3f(0.0f,1.0f,0.0f);                // set color to green
      glVertex3f(-50.0f, -50.0f, 50.0f);
      glColor3f(1.0f,1.0f,1.0f);                // set color to white
      glVertex3f( -50.0f, -50.0f, -50.0f);
      glColor3f(0.0f,0.0f,1.0f);                // set color to blue
      glVertex3f( 50.0f, -50.0f, -50.0f);
      glColor3f(1.0f,1.0f,0.0f);                // set color to yellow
      glVertex3f( 50.0f, -50.0f, 50.0f);
   glEnd();

      glLoadIdentity();
      glTranslatef(0.0,80.0,0.0);
      glRotatef(alpha,0.0f,1.0f,0.0f);    // rotate the text if you like
      glScalef(20.0,20.0,20.0);
      glColor3f(fabs(sin(0.001*alpha)),fabs(cos(0.001*beta)),fabs(tan(0.001*beta)));
      glListBase(nFontList);
      glCallLists (11, GL_UNSIGNED_BYTE, "Text Output");


}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormPaint(TObject *Sender)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    DrawObjects();
    glFlush();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormDestroy(TObject *Sender)
{
    wglMakeCurrent(NULL, NULL);
    wglDeleteContext(hrc);
}

